"""Imports Crypto PublicKey

Now you can do:
    >>> from CryptoPlus.PublicKey import *
OR:
    >>> from CryptoPlus.PublicKey import RSA
but not:
    >>> import CryptoPlus.PublicKey.RSA
"""

from Crypto.PublicKey import *
